import requests
import json

from ..common.constants import STATUS_UPDATE_ENDPOINT_PREFIX, TIYARO_API_KEY, MODEL_VERSION
from ..common.utils import get_tiyaro_api_key, failed
from ..version import __version__

C_STATUS_INIT = 'init'
C_STATUS_TEST_FAILED = 'test_failed'
C_STATUS_TEST_PASSED = 'test_passed'
C_STATUS_PUSH = 'pushed'
C_STATUS = 'status'
C_FRAMEWORK = 'framework'

STATUS_UPDATE_API = '{}/{}/{}'


def update_status_init(model_name, model_framework):
    _update_status(C_STATUS_INIT, model_name, model_framework)
    pass


def update_status_test_passed(model_name):
    _update_status(C_STATUS_TEST_PASSED, model_name)
    pass


def update_status_test_failed(model_name):
    _update_status(C_STATUS_TEST_FAILED, model_name)
    pass


def update_status_pushed(model_name):
    _update_status(C_STATUS_PUSH, model_name)
    pass


def _update_status(status, name, framework=None):
    url = STATUS_UPDATE_API.format(
        STATUS_UPDATE_ENDPOINT_PREFIX, name, MODEL_VERSION)
    token = get_tiyaro_api_key()
    headers = {
        'Authorization': f'Bearer {token}',
        'cli_version': __version__
    }
    body = {}
    body[C_STATUS] = status
    body[C_FRAMEWORK] = framework
    resp = requests.post(url, json=body, headers=headers)
    if resp.status_code == 401:
        failed(
            f'Token Authorization Error.  Is your {TIYARO_API_KEY} still valid ?')
        exit(-1)
    if resp.status_code == 400:
        body = json.loads(resp.content)
        if body['code'] == 'UPGRADE_REQUIRED':
            failed(
                f'Please upgrade your tiyaro cli.  Supported version: {body["message"]}')
            exit(-1)
    if resp.status_code != 200:
        failed(f'Unable to udpate status: {resp.content}')



from marshmallow import Schema, class_registry
from abc import ABC, abstractmethod


class TiyaroBase(ABC):
    def __init__(self) -> None:
        self.input_schema = None
        self.output_schema = None

    def def_input_schema(self, schemaDict):
        self.input_schema = Schema.from_dict(schemaDict, name="InputSchema")
        class_registry.register("InputSchema", self.input_schema)

    def def_output_schema(self, schemaDict):
        self.output_schema = Schema.from_dict(schemaDict, name="OutputSchema")
        class_registry.register("OutputSchema", self.output_schema)

    def declare_schema(self):
        pass

    @abstractmethod
    def setup_model(self, pretrained_file_path):
        pass

    @abstractmethod
    def infer(self, json_input):
        pass


import click

from .cmds.clean import clean
from .cmds.infer import infer
from .cmds.init import init
from .cmds.push import push
from .cmds.test import test


@click.group()
def cli():
    """
    tiyaro - Tiyaro Cli
    """
    pass


cli.add_command(init)
cli.add_command(push)
cli.add_command(test)
cli.add_command(infer)
cli.add_command(clean)



import json
import os
import shutil

import click
import requests

from ..api.status_update import update_status_pushed
from ..handler.cli_state import get_model_name_with_suffix
from ..common.constants import *
from ..common.utils import (failed, get_model_endpoint, get_tiyaro_api_key,
                            success, warn, delete_test_venv, get_python_version)
from ..handler.cli_state import get_model_name_with_suffix, get_model_framework, get_model_class
from ..handler.tiyaro_test_state import get_test_state, is_test_attempted
from ..handler.utils import validate_handler_exists
from ..handler.model_manifest import get_pretrained_file_or_url, validate_manifest_mandatory_params
from ..version import __version__


@click.command()
@click.option('-f', '--force', is_flag=True, default=False, help=VERBOSE_HELP)
@click.option('-v', '--verbose', is_flag=True, default=False, help=VERBOSE_HELP)
def push(force, verbose):
    """
    - Pushes model repo to Tiyaro Infrastructure
    """
    do_push(force, verbose)


def push_model(tiyaro_jwt_token: str, is_verbose):
    tiyaro_model_name = get_model_name_with_suffix()
    tiyaro_model_version = MODEL_VERSION

    def tiyaro_get_name_reservation_record(api_token):
        headers = {
            "Authorization": f"Bearer {api_token}"
        }

        resp = requests.get(url=NAME_RESERVATION_ENDPOINT, headers=headers)
        if resp.status_code not in (200, 201):
            failed(
                f"Failed to reserve namespace.  Error code on {NAME_RESERVATION_ENDPOINT}: STATUS {resp.status}, REASON {resp.content}")
            warn('Kindly reach out to Tiyaro Support Team')
            exit(-1)
        return json.loads(resp.content)

    tiyaro_model_vendor = tiyaro_get_name_reservation_record(tiyaro_jwt_token)[
        'vendor_name']

    # TODO - USING PYTHON
    os.system(f"mkdir -p {UPLOAD_ARTIFACT_DIR}")
    # os.system("cp ./.tiyaro-install/state.json ./build") TODO: MULTI MODELS
    os.system(
        f"git archive --output={UPLOAD_ARTIFACT_DIR}/{GIT_SNAPSHOT_FILE_NAME} HEAD .")

    pretrained_file_path = get_pretrained_file_or_url()
    name_with_ext = None
    if os.path.isfile(pretrained_file_path):
        name_with_ext = os.path.basename(pretrained_file_path)
        dest_file_path = f'{UPLOAD_ARTIFACT_DIR}/{name_with_ext}'
        success(
            f'DEBUG - copying pretrained file from: {pretrained_file_path} to {dest_file_path}', is_verbose)
        shutil.copyfile(pretrained_file_path, dest_file_path)

    package_for_upload_client(UPLOAD_ARTIFACT_DIR, tiyaro_model_name,
                              tiyaro_model_vendor, tiyaro_model_version, tiyaro_jwt_token, name_with_ext, is_verbose)

    success(
        f'\n***Uploaded {tiyaro_model_name} to be converted onto Tiyaro Infrastructure', is_verbose)
    return get_model_endpoint(tiyaro_model_vendor, tiyaro_model_version, tiyaro_model_name)


def package_for_upload_client(build_subdir, tiyaro_model_name, tiyaro_model_vendor, model_version, tiyaro_jwt_token, pretrained_file_name, is_verbose):
    tiyaro_test_state = get_test_state()
    # TODO: Also send git hash info
    manifest_json = {
        'tiyaro_job_descriptor': TIYARO_JOB_DESCRIPTOR,
        'env_TIYARO_PUSH_NAME_BASE': None,
        'model_name': tiyaro_model_name,
        'model_vendor': tiyaro_model_vendor,
        'version': model_version,
        'model_type': TIYARO_MODEL_TYPE,
        'pretrained_file_name': pretrained_file_name,
        'tiyaro_authorization_token': tiyaro_jwt_token,
        C_MODEL_FRAMEWORK: get_model_framework(),
        C_MODEL_CLASS: get_model_class(),
        C_TEST_INPUT: tiyaro_test_state.get(C_TEST_INPUT, None),
        C_TEST_OUTPUT: tiyaro_test_state.get(C_TEST_OUTPUT, None),
        C_TEST_USER_CONFIRMED: tiyaro_test_state.get(C_TEST_USER_CONFIRMED, False),
        'user_python_env': get_python_version(),
        'cli_version': __version__
    }

    # Need to decorate with some tensor-specific information
    # with signature input and output

    manifest_json_filename = '/'.join([build_subdir, JSON_MANIFEST_FILE_NAME])
    with open(manifest_json_filename, 'w') as fout:
        fout.write(json.dumps(manifest_json, indent=4))

    success('START - Tarballing the artifacts into a single file', is_verbose)
    os.system(f'tar cf - {build_subdir} | gzip > {FILE_TO_UPLOAD}')
    success('FINISH - Tarballing the artifacts into a single file', is_verbose)

    success('START - Requesting a presigned url for upload', is_verbose)
    response_presigned_request = requests.get(
        PRESIGNED_REQUEST_ENDPOINT,
        headers={
            'Authorization': tiyaro_jwt_token
        })
    if response_presigned_request.status_code == 401:
        failed(
            f'Token Authorization Error.  Is your {TIYARO_API_KEY} still valid ?')
        exit(-1)
    elif response_presigned_request.status_code != 200:
        failed(
            f'Failed to received the presigned url from API endpoint {PRESIGNED_REQUEST_ENDPOINT}.', is_verbose)
        exit(-1)

    response_presigned_request_json = response_presigned_request.json()
    url = response_presigned_request_json['post_url']
    data = response_presigned_request_json['post_data_required']
    bucket_name = url.split('//')[1].split('.')[0]
    bucket_key = data['key']
    # click.echo(f'INFO - Received a presigned for S3 bucket {bucket_name} and key {bucket_key}')
    success(f'FINISH - Received a presigned url for upload {url}', is_verbose)

    size_payload = os.path.getsize(FILE_TO_UPLOAD)
    success(
        f'START - Upload of payload size {str(round(size_payload / (1024 * 1024), 2))}MB to a presigned url')
    with open(FILE_TO_UPLOAD, 'rb') as test_file_to_upload:
        # the key supposed to be file may be
        files = {'file': test_file_to_upload}
        response_file_upload = requests.post(url, data=data, files=files)
    if response_file_upload.status_code != 204:
        failed(
            f'Failed to upload the {FILE_TO_UPLOAD} to presigned url')
        exit(-1)

    success(f'FINISH - Upload to a presigned url')
    os.remove(FILE_TO_UPLOAD)
    return bucket_name, bucket_key

# before pushing, clean unwanted tiyaro generated artifacts


def clean_tiyaro_generated(is_verbose):
    delete_test_venv(is_verbose)


def do_push(is_force_push, is_verbose):
    clean_tiyaro_generated(is_verbose)
    validate_handler_exists()
    validate_manifest_mandatory_params()

    if not is_force_push:
        test_attempted = is_test_attempted()
        if not test_attempted:
            warn("Did you try 'tiyaro test' to test your handler ?\n You can also use 'tiyaro push -f' to force push without testing")
            exit(-1)

    token = get_tiyaro_api_key()
    api = push_model(token, is_verbose)
    update_status_pushed(get_model_name_with_suffix())

    success('\n\nView status at:')
    success(STATUS_PAGE_URL)
    success('\nInference API:')
    success(api)



import os
import shutil
import platform

import click

from .constants import (INFERENCE_ENDPOINT_PREFIX, TIYARO_TEST_VENV_PATH,
                        TIYARO_API_KEY)


def wip():
    warn(f'** WORK IN PROGRESS **\n Please wait for new version release..')


def get_tiyaro_api_key():
    token = os.getenv(TIYARO_API_KEY, None)
    if token is None:
        failed(f'Please set {TIYARO_API_KEY} env var')
        exit(-1)
    return token


def get_model_endpoint(vendor, version, model):
    return f'{INFERENCE_ENDPOINT_PREFIX}/{vendor}/{version}/{model}'


def success(msg, is_verbose=True):
    if is_verbose:
        click.secho(msg, fg='green')


def failed(msg, is_verbose=True):
    if is_verbose:
        click.secho(msg, fg='red')


def warn(msg, is_verbose=True):
    if is_verbose:
        click.secho(msg, fg='yellow')


def delete_test_venv(is_verbose=True):
    if (os.path.isdir(TIYARO_TEST_VENV_PATH)):
        success(f'Cleaning up VENV: {TIYARO_TEST_VENV_PATH}', is_verbose)
        shutil.rmtree(TIYARO_TEST_VENV_PATH)


def get_python_version():
    return platform.python_version()



import os

import yaml
import validators

from ..common.constants import HANDLER_MODEL_MANIFEST_FILE, MODEL_VERSION, C_PRETRAINED_FILE_OR_URL, C_REQUIREMENTS_FILE, C_MODEL_FILE_PATH
from ..common.utils import failed
from .utils import validate_model_manifest_file_exists


# TODO: reintroduce when we add support for version
# def get_model_version():
#     validate_model_manifest_file_exists()
#     return f'{MODEL_VERSION}'

# # use this only for getting value from manifest file
# def get_model_name():
#     validate_model_manifest_file_exists()
#     return _get_param('name')

def validate_manifest_mandatory_params():
    get_pretrained_file_or_url()
    get_requirements_file_path()
    validate_model_file_path()

def validate_model_file_path():
    validate_model_manifest_file_exists()
    file = _get_param(C_MODEL_FILE_PATH)
    if not os.path.isfile(file):
        failed(
            f'Invalid file path: {file} key:{C_MODEL_FILE_PATH} in {HANDLER_MODEL_MANIFEST_FILE}')
        exit(-1)

def get_pretrained_file_or_url():
    validate_model_manifest_file_exists()
    file_or_url = _get_param(C_PRETRAINED_FILE_OR_URL)
    if not validators.url(file_or_url) and not os.path.isfile(file_or_url):
        failed(
            f'Invalid file path or url: {file_or_url} key:{C_PRETRAINED_FILE_OR_URL} in {HANDLER_MODEL_MANIFEST_FILE}')
        exit(-1)
    return file_or_url

def get_requirements_file_path():
    validate_model_manifest_file_exists()
    path = _get_param(C_REQUIREMENTS_FILE)
    if not os.path.isfile(path):
        failed(
            f'Invalid file path: {path} key:{C_REQUIREMENTS_FILE} in {HANDLER_MODEL_MANIFEST_FILE}')
        exit(-1)
    return path


def _get_param(field):
    with open(HANDLER_MODEL_MANIFEST_FILE, 'r') as file:
        contents = yaml.safe_load(file)
        value = contents[field]
        if (
            (value is None)
            or (not isinstance(value, str))
            or (not value.strip())
        ):
            failed(
                f'Please set a valid model {field} in {HANDLER_MODEL_MANIFEST_FILE}')
        return value



import os
import shutil
import subprocess
import tarfile
import re
import glob

import click
import requests

from ..api.status_update import update_status_init
from ..common.constants import *
from ..common.utils import failed, get_tiyaro_api_key, success, warn
from ..handler.cli_state import get_model_name_with_suffix, save_model_metadata, get_model_framework


@click.command()
@click.option('-f', '--force', is_flag=True, default=False, help=INIT_HELP)
@click.option('-v', '--verbose', is_flag=True, default=False, help=VERBOSE_HELP)
def init(force, verbose):
    """
    - Initializes model repo for Tiyaro Push
    """
    get_tiyaro_api_key()
    validate_ok_to_init(force, verbose)

    name = get_name(verbose)
    framework = get_framework(verbose)
    clazz = get_model_class(verbose)
    save_model_metadata(name, framework, clazz, verbose)
    update_status_init(get_model_name_with_suffix(), get_model_framework())

    if framework == 'pytorch':
        get_pytorch_templates(clazz, verbose)
        success(
            f'Created {TIYARO_HANDLER_DIR}, {HANDLER_MODEL_MODEL_TEST_FILE} templates successfully !')
    else:
        warn('Thank you for your interest in trying Tiyaro Cli !  Currently, only pytorch framework is supported.  Kindly reachout to Tiyaro Team.')
        subprocess.run(CMD_TIYARO_FORCE_CLEAN, shell=True)


def get_name(is_verbose):
    PATTERN = "[a-zA-Z_][a-zA-Z0-9_]*"
    while True:
        name = click.prompt('Please enter the name of your model', type=str)
        if re.fullmatch(PATTERN, name):
            break
        else:
            failed(f'Invalid name.  Must match pattern "{PATTERN}"\n')
    return name


def get_framework(is_verbose):
    while True:
        framework_opt = '\n 1\t Pytorch\n 2\t Tensorflow\n 3\t JAX\n 4\t Other -specify\n'
        option = click.prompt(
            f'Please enter the framework of your model {framework_opt} \t\t\t', type=str)
        option = option.casefold()

        if option in ['1', 'pytorch']:
            option = 'pytorch'
            break
        elif option in ['2', 'tensorflow']:
            option = 'tensorflow'
            break
        elif option in ['3', 'jax']:
            option = 'jax'
            break
        elif option == '4':
            failed(f'For option 4, please specify the framework name')
        else:
            break

        success(f'DEBUG - user selected framework is: {option}', is_verbose)
    return option


def get_model_class(is_verbose):
    while True:
        model_type_opt = f'\n 1\t {CLASS_IMAGE_CLASSIFICATION}\n 2\t {CLASS_TEXT_CLASSIFICATION}\n 3\t Other -{CLASS_CUSTOM}\n'
        option = click.prompt(
            f'Please enter the model type {model_type_opt} \t\t\t', type=str)
        option = option.casefold()

        if option in ['1', CLASS_IMAGE_CLASSIFICATION]:
            option = CLASS_IMAGE_CLASSIFICATION
            break
        elif option in ['2', CLASS_TEXT_CLASSIFICATION]:
            option = CLASS_TEXT_CLASSIFICATION
            break
        elif option in ['3', CLASS_CUSTOM]:
            option = CLASS_CUSTOM
            break
        else:
            failed(f'Please specify valid option')

        success(f'DEBUG - user selected class is: {option}', is_verbose)
    return option


def validate_ok_to_init(is_overwrite, is_verbose):
    __init_validate(is_overwrite, HANDLER_MODEL_MANIFEST_FILE)
    __init_validate(is_overwrite, HANDLER_MODEL_MODEL_HANDLER_FILE)
    __init_validate(is_overwrite, HANDLER_MODEL_MODEL_TEST_FILE)


def __init_validate(is_overwrite, file):
    if (os.path.isfile(file)):
        if not is_overwrite:
            warn(f"{file} already exists.  To force init kindly use 'tiyaro init -f' ")
            exit(-1)
        else:
            subprocess.run(CMD_TIYARO_FORCE_CLEAN, shell=True)


def get_pytorch_templates(clazz, is_verbose):
    success('Fetching tiyaro pytorch handler templates...')
    token = get_tiyaro_api_key()
    resp = requests.get(
        f'{PUSH_SUPPORT_FILES_ENDPOINT}/{ARTIFACTS_FILE_NAME}',
        headers={
            'Authorization': token
        })
    if resp.status_code == 200:
        template_url = resp.content
    else:
        failed(resp.status_code)
        failed(resp.content)
        failed(
            f'Unable to get templates URL.  Is your {TIYARO_API_KEY} still valid ?')
        exit(-1)

    os.makedirs(ARTIFACTS_DOWNLOAD_DIR)
    downloaded_artifact = f'{ARTIFACTS_DOWNLOAD_DIR}/ARTIFACTS_FILE_NAME'

    resp = requests.get(template_url, stream=True)
    if resp.status_code == 200:
        with open(downloaded_artifact, 'wb') as f:
            f.write(resp.raw.read())
    else:
        failed(
            f'Unable to get templates.  Is your {TIYARO_API_KEY} still valid ?')
        exit(-1)

    def members(tf, sub_folder):
        l = len(sub_folder)
        for member in tf.getmembers():
            if member.path.startswith(sub_folder):
                member.path = member.path[l:]
                yield member

    tar = tarfile.open(downloaded_artifact)
    tar.extractall(path=TIYARO_HANDLER_DIR,
                   members=members(tar, ARTIFACTS_FILES_DIR))
    tar.close()  # close before rmtree, some filesystems (nfs) may fail deleting dir with open files
    filter_class_specific_templates(clazz, is_verbose)
    # move test file to project root for tiyaro test
    shutil.move(f'{TIYARO_HANDLER_DIR}/{HANDLER_MODEL_MODEL_TEST_FILE}',
                f'{HANDLER_MODEL_MODEL_TEST_FILE}')
    shutil.rmtree(ARTIFACTS_DOWNLOAD_DIR)


def filter_class_specific_templates(clazz, is_verbose):
    to_rename = class_handler_map[clazz]
    old_name = to_rename
    sub_str = os.path.splitext(os.path.basename(to_rename))[0]
    to_rename = to_rename.replace(sub_str, MODEL_HANDLER)
    warn(
        f'DEBUG - for class: {clazz} renaming template : {old_name} to {to_rename}', is_verbose)
    os.rename(old_name, to_rename)
    
    for f in glob.glob(HANDLER_MODEL_CLASS_TEMPLATES_PATTERN):
        warn(f'DEBUG - deleting {f}', is_verbose)
        os.remove(f)



import json
import os
import shutil

import requests
import validators
import yaml

from ...common.constants import (C_PRETRAINED_FILE_OR_URL, CLASS_CUSTOM,
                                 CLASS_IMAGE_CLASSIFICATION,
                                 CLASS_TEXT_CLASSIFICATION,
                                 HANDLER_MODEL_MANIFEST_FILE,
                                 KEYWORD_FROM_CONFIG, TEST_COMMAND_ARG_INPUT,
                                 TEST_COMMAND_ARG_OUTPUT,
                                 TEST_COMMAND_ARG_PRETRAINED,
                                 TEST_COMMAND_ARG_VERBOSE)
from ...common.utils import warn
from ...handler.cli_state import get_model_class
from ...handler.tiyaro_test_state import save_input, save_output
from ..base_handler import TiyaroBase

KEYWORD_FROM_CONFIG = 'from-config'
HANDLER_MODEL_MANIFEST_FILE = './tiyaro_handler/model_manifest.yml'
C_PRETRAINED_FILE_OR_URL = 'pretrained_model_url'
TEST_COMMAND_ARG_INPUT
TEST_COMMAND_ARG_OUTPUT
TEST_COMMAND_ARG_PRETRAINED
TEST_COMMAND_ARG_VERBOSE

def get_pretrained_file_path(x):
    def local_file(x, err_msg):
        if os.path.isfile(x):
            return x
        else:
            raise ValueError(err_msg)

    if x == KEYWORD_FROM_CONFIG:
        # read from yaml
        # if url, download to /tmp and return path
        # if local path, return path
        with open(HANDLER_MODEL_MANIFEST_FILE, 'r') as file:
            contents = yaml.safe_load(file)
            value = contents[C_PRETRAINED_FILE_OR_URL]
            if (
                (value is None)
                or (not isinstance(value, str))
                or (not value.strip())
            ):
                raise ValueError(
                    f'Please set a valid model {C_PRETRAINED_FILE_OR_URL} in {HANDLER_MODEL_MANIFEST_FILE}')
            if validators.url(value):
                r = requests.get(value, stream=True)
                file_path = '/tmp/pre_trained.pth'
                print(
                    f'DOWNLOADING - pretrained file to {file_path} from URL: {value}')
                if r.status_code == 200:
                    with open(file_path, 'wb') as f:
                        r.raw.decode_content = True
                        shutil.copyfileobj(r.raw, f)
                    print('DOWNLOAD - Done')
                    return file_path
                else:
                    raise RuntimeError(
                        f'Unable to download pretrained file from: {value}')
            else:
                return local_file(value, 'Expected valid local file path')

    return local_file(x, 'Expected valid local file path')


def _get_input_json(x, is_verbose):
    warn(f'DEBUG - input is of type: {type(x)}, input:\n {x}', is_verbose)
    if os.path.isfile(x):
        with open(x, 'r') as f:
            return json.load(f)
    if isinstance(x, str):
        return json.loads(x)

    raise ValueError('Expected Valid JSON input string or file path')


def _get_input_image_classification(x, is_verbose):
    warn(f'DEBUG - input is of type: {type(x)}, input:\n {x}', is_verbose)
    if os.path.isfile(x):
        with open(x, 'rb') as img:
            return img.read()

    raise ValueError('Expected path to valid image file')


def _get_input_text_classification(x, is_verbose):
    warn(f'DEBUG - input is of type: {type(x)}, input:\n {x}', is_verbose)
    if isinstance(x, str):
        return x

    raise ValueError('Expected input string')


def get_input_by_model_type(x, is_verbose):
    clazz = get_model_class()
    if clazz == CLASS_CUSTOM:
        return _get_input_json(x, is_verbose)
    elif clazz == CLASS_IMAGE_CLASSIFICATION:
        return _get_input_image_classification(x, is_verbose)
    elif clazz == CLASS_TEXT_CLASSIFICATION:
        return _get_input_text_classification(x, is_verbose)

    raise ValueError(
        f'ERROR - cannot get input for unknown model-type: {clazz}')


def validate_and_save_test_input(handler: TiyaroBase, test_input, is_verbose):
    warn(f'DEBUG - input is of type: {type(test_input)}', is_verbose)

    clazz = get_model_class()
    if clazz == CLASS_CUSTOM:
        if handler.input_schema:
            # we validate if schema is declared
            handler.input_schema().load(test_input)
            save_input(test_input)
            print(f'INPUT - Validation Done')
        else:
            print('WARN - Input schema not defined')
    elif clazz == CLASS_IMAGE_CLASSIFICATION:
        if not isinstance(test_input, (bytearray, bytes)):
            raise ValueError(
                f'ERROR - for model-type: {CLASS_IMAGE_CLASSIFICATION}, input must be of type: bytes or bytearray')
        save_input(test_input)
    elif clazz == CLASS_TEXT_CLASSIFICATION:
        if not isinstance(test_input, str):
            raise ValueError(
                f'ERROR - for model-type: {CLASS_TEXT_CLASSIFICATION}, input must be of type: {type("string")}')
        save_input(test_input)
    else:
        print(f'WARN - unknown model-type: {clazz}')


def validate_and_save_test_output(handler: TiyaroBase, test_output, is_verbose):
    warn(f'DEBUG - output is of type: {type(test_output)}', is_verbose)

    clazz = get_model_class()
    if clazz == CLASS_CUSTOM:
        if handler.output_schema:
            # we validate if schema is declared
            handler.output_schema().load(test_output)
            save_output(test_output)
            print(f'OUTPUT - Validation Done')
        else:
            print('WARN - Output schema not defined')
    elif clazz == CLASS_IMAGE_CLASSIFICATION:
        if not isinstance(test_output, dict):
            raise ValueError(
                f'ERROR - for model-type: {CLASS_IMAGE_CLASSIFICATION}, output must be of type: {type({})}')
        save_output(test_output)
    elif clazz == CLASS_TEXT_CLASSIFICATION:
        if not isinstance(test_output, dict):
            raise ValueError(
                f'ERROR - for model-type: {CLASS_TEXT_CLASSIFICATION}, output must be of type: {type({})}')
        save_output(test_output)
    else:
        print(f'WARN - unknown model-type: {clazz}')



import subprocess
import venv
import os

import click

from ..api.status_update import (update_status_test_failed,
                                 update_status_test_passed)
from ..handler.cli_state import get_model_name_with_suffix
from ..common.constants import (HANDLER_MODEL_MANIFEST_FILE,
                                HANDLER_MODEL_MODEL_TEST_FILE, TEST_COMMAND_ARG_INPUT, TEST_COMMAND_ARG_OUTPUT, TEST_COMMAND_ARG_PRETRAINED, TEST_COMMAND_ARG_VERBOSE, VERBOSE_HELP, TIYARO_TEST_VENV_PATH, KEYWORD_FROM_CONFIG)
from ..common.utils import failed, success, delete_test_venv
from ..handler.model_manifest import get_requirements_file_path, validate_manifest_mandatory_params
from ..handler.tiyaro_test_state import save_user_confirmed
from ..handler.utils import (validate_handler_exists,
                             validate_handler_test_file_exists)


@click.command()
@click.option('-p', TEST_COMMAND_ARG_PRETRAINED, required=False, help=f'pretrained_file path or url.  default is value from {HANDLER_MODEL_MANIFEST_FILE}')
@click.option('-i', TEST_COMMAND_ARG_INPUT, required=True, help=f'Valid input string or file path that contains input in expected format for your model-type')
@click.option('-o', TEST_COMMAND_ARG_OUTPUT, required=False, help=f'Inference output file - saves inference output to the specified file')
@click.option('-dv', '--delvenv', is_flag=True, default=True, help=f'Delete venv after each tiyaro test run, default=True')
@click.option('-v', TEST_COMMAND_ARG_VERBOSE, is_flag=True, default=False, help=VERBOSE_HELP)
def test(pretrained, input, output, delvenv, verbose):
    """
    - Test model locally & validate expected input output formats
    """
    do_test(pretrained, input, output, delvenv, verbose)


def do_test(pretrained, input, output_file, is_del_venv, is_verbose):
    validate_handler_exists()
    validate_handler_test_file_exists()
    validate_manifest_mandatory_params()

    requirements_file_path = get_requirements_file_path()

    success(f'Creating VENV: {TIYARO_TEST_VENV_PATH}')
    venv.create(TIYARO_TEST_VENV_PATH, with_pip=True)

    os.system(
        f'{TIYARO_TEST_VENV_PATH}/bin/pip3 install -r {requirements_file_path}')

    if not pretrained:
        pretrained = KEYWORD_FROM_CONFIG
    
    # input can have space so added quotes
    p = subprocess.run(
        f'{TIYARO_TEST_VENV_PATH}/bin/python {HANDLER_MODEL_MODEL_TEST_FILE} {TEST_COMMAND_ARG_PRETRAINED} {pretrained} {TEST_COMMAND_ARG_INPUT}="{input}" {TEST_COMMAND_ARG_OUTPUT} {output_file} {TEST_COMMAND_ARG_VERBOSE} {is_verbose}', shell=True)

    if is_del_venv:
        delete_test_venv(is_verbose)

    if p.returncode == 0:
        while True:
            yes_or_no = click.prompt(
                'Is this your expected output (Y/n) ?', type=str)
            if yes_or_no in ["Y", 'n']:
                break
        if yes_or_no == "Y":
            save_user_confirmed(True)
        else:
            save_user_confirmed(False)

        update_status_test_passed(get_model_name_with_suffix())
        success('Test successful !  You can push your model now.')
    else:
        update_status_test_failed(get_model_name_with_suffix())
        failed('Test failed !')



import os

from ..common.constants import *
from ..common.utils import failed

def validate_handler_exists():
    if (not os.path.exists(HANDLER_MODEL_MANIFEST_FILE)
            or not os.path.exists(HANDLER_MODEL_MODULE_FILE)
            or not os.path.exists(HANDLER_MODEL_MODEL_HANDLER_FILE)
            ):
        failed(
            f'Missing mandatory files in {TIYARO_HANDLER_DIR} directory.  Initialize using tiyaro init')
        exit(-1)


def validate_model_manifest_file_exists():
    if (not os.path.exists(HANDLER_MODEL_MANIFEST_FILE)):
        failed(
            f'Missing mandatory model handler file: {HANDLER_MODEL_MANIFEST_FILE}.  Initialize using tiyaro init')
        exit(-1)


def validate_handler_test_file_exists():
    if (not os.path.exists(HANDLER_MODEL_MODEL_TEST_FILE)):
        failed(
            f'Missing mandatory handler test file: {HANDLER_MODEL_MODEL_TEST_FILE}.  Initialize using tiyaro init')
        exit(-1)



import os
import shutil

import click

from ..common.constants import *
from ..common.utils import success, warn


@click.command()
@click.option('-f', '--force', is_flag=True, help=CLEAN_HELP)
@click.option('-v', '--verbose', is_flag=True, default=False, help=VERBOSE_HELP)
def clean(force, verbose):
    """
    - Forcefully cleans all tiyaro generated files
    """
    if not force:
        warn('No action taken.  Run using -f flag')
        exit(-1)
    do_clean(verbose)


def do_clean(is_verbose):
    took_action = False
    if (os.path.isdir(TIYARO_HANDLER_DIR)):
        shutil.rmtree(TIYARO_HANDLER_DIR)
        took_action = True

    if (os.path.isfile(HANDLER_MODEL_MODEL_TEST_FILE)):
        os.remove(HANDLER_MODEL_MODEL_TEST_FILE)
        took_action = True

    if took_action:
        success(
            f'Cleaned {TIYARO_HANDLER_DIR}, {HANDLER_MODEL_MODEL_TEST_FILE} templates', is_verbose)



import os
import json
import base64

from ..common.constants import TIYARO_INTERNAL_DIR, TIYARO_INTERNAL_TEST_STATE_FILE, C_TEST_INPUT, C_TEST_OUTPUT, C_TEST_USER_CONFIRMED


def save_input(test_input):
    state = get_test_state()
    if isinstance(test_input, (bytearray, bytes)):
        test_input = base64.b64encode(test_input).decode('utf-8')
    state[C_TEST_INPUT] = test_input
    _write_state(state)
    pass


def save_output(test_output):
    state = get_test_state()
    state[C_TEST_OUTPUT] = test_output
    _write_state(state)
    pass


def save_user_confirmed(is_user_confirmed):
    state = get_test_state()
    state[C_TEST_USER_CONFIRMED] = is_user_confirmed
    _write_state(state)
    pass

def is_test_attempted():
    state = get_test_state()
    test_input = state.get(C_TEST_INPUT, None)
    return True if test_input else False

def get_test_state():
    if (not os.path.exists(TIYARO_INTERNAL_TEST_STATE_FILE)):
        existing_state = {}
    else:
       with open(TIYARO_INTERNAL_TEST_STATE_FILE, "r") as f:
           existing_state = json.load(f)
    return existing_state


def _write_state(state):
    if not os.path.isdir(TIYARO_INTERNAL_DIR):
        os.makedirs(TIYARO_INTERNAL_DIR)
    with open(TIYARO_INTERNAL_TEST_STATE_FILE, 'w+', encoding='utf-8') as f:
        json.dump(state, f, ensure_ascii=False, indent=4)



from setuptools import find_packages, setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
c_dict = {}
with open(this_directory / "tiyaro/version.py") as ver_file:
    exec(ver_file.read(), c_dict)

# https://packaging.python.org/en/latest/guides/making-a-pypi-friendly-readme/
setup(
    name='tiyaro',
    version=c_dict['__version__'],
    description='Tiyaro Python Package Index',
    long_description=long_description,
    long_description_content_type='text/markdown',

    author='Venkat Raman, I-Jong Lin',
    author_email='venkat@tiyaro.ai, ijonglin@tiyaro.ai',

    url='https://github.com/pypa/sampleproject',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent'],
    license='Apache License 2.0',
    license_files=('LICENSE',),
    platforms='any',

    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click==8.0.4',
        'requests',
        'PyYAML',
        'marshmallow',
        'validators'
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'tiyaro = tiyaro.main:cli',
        ],
    },
)



import json
import os
import uuid

from ..common.constants import *
from ..common.utils import failed, success
from ..api.status_update import update_status_init

C_MODEL_NAME = 'model_name'
C_MODEL_NAME_SUFFIX = 'model_name_suffix'


def get_model_name_with_suffix():
    return '{}_{}'.format(get_model_name(), get_model_suffix())


def get_model_name():
    return _get_state(C_MODEL_NAME)


def get_model_suffix():
    return _get_state(C_MODEL_NAME_SUFFIX)


def get_model_framework():
    return _get_state(C_MODEL_FRAMEWORK)


def get_model_class():
    return _get_state(C_MODEL_TYPE)


def save_model_metadata(model_name, model_framework, clazz, is_verbose):
    state = _get_full_state()
    state[C_MODEL_NAME] = model_name
    state[C_MODEL_FRAMEWORK] = model_framework
    state[C_MODEL_TYPE] = clazz
    suffix = state.get(C_MODEL_NAME_SUFFIX, None)
    if not suffix:
        state[C_MODEL_NAME_SUFFIX] = __rand()
    _write_state(state)
    pass


def _validate_state_exists():
    if (not os.path.exists(TIYARO_INTERNAL_STATE_FILE)):
        failed(f'{TIYARO_INTERNAL_STATE_FILE} doesnot exist')
        exit(-1)


def _get_full_state():
    if (not os.path.exists(TIYARO_INTERNAL_STATE_FILE)):
        existing_state = {}
    else:
        with open(TIYARO_INTERNAL_STATE_FILE, "r") as f:
            existing_state = json.load(f)
    return existing_state


def _get_state(key):
    _validate_state_exists()
    state = _get_full_state()
    value = state[key]
    if not value and value.strip():
        failed(f'Invalid key: {key}, value: {value}')
        exit(-1)
    return value


def _write_state(state):
    if not os.path.isdir(TIYARO_INTERNAL_DIR):
        os.makedirs(TIYARO_INTERNAL_DIR)
    with open(TIYARO_INTERNAL_STATE_FILE, 'w+', encoding='utf-8') as f:
        json.dump(state, f, ensure_ascii=False, indent=4)


def create_internal_dir(is_verbose):
    if not os.path.isdir(TIYARO_INTERNAL_DIR):
        os.makedirs(ARTIFACTS_DOWNLOAD_DIR)
    else:
        success(f'{TIYARO_INTERNAL_DIR} already exists', is_verbose)


def __rand(string_length=6):
    random = str(uuid.uuid4())
    return random[0:string_length]



# ENDPOINTS
PUSH_SUPPORT_FILES_ENDPOINT = 'https://wpxkueck1a.execute-api.us-west-2.amazonaws.com/Prod/customer_download'
NAME_RESERVATION_ENDPOINT = 'https://wpxkueck1a.execute-api.us-west-2.amazonaws.com/Prod/registered_vendor_name'
PRESIGNED_REQUEST_ENDPOINT = 'https://wpxkueck1a.execute-api.us-west-2.amazonaws.com/Prod/hello'
STATUS_UPDATE_ENDPOINT_PREFIX = 'https://1quz3qvhsi.execute-api.us-west-2.amazonaws.com/Prod/push-model'
INFERENCE_ENDPOINT_PREFIX = 'https://api.tiyaro.ai/v1/ent'

# SHARED CONSTANTS
TIYARO_API_KEY = 'TIYARO_API_KEY'
TIYARO_JOB_DESCRIPTOR = 'GIT_PYTORCH_SNAPSHOT'
TIYARO_MODEL_TYPE = 'GIT_PASSTHROUGH'
MODEL_VERSION = '1.0'

VERBOSE_HELP = f'Display more logs'

# INIT CONSTANTS
TIYARO_HANDLER_DIR = './tiyaro_handler'
ARTIFACTS_DOWNLOAD_DIR = f'{TIYARO_HANDLER_DIR}/artifacts'
ARTIFACTS_FILE_NAME = 'tiyaro_handler_templates.tgz'
ARTIFACTS_FILES_DIR = 'staging/tiyaro_handler/'
MODEL_HANDLER = 'model_handler'
HANDLER_MODEL_MANIFEST_FILE = f'{TIYARO_HANDLER_DIR}/model_manifest.yml'
HANDLER_MODEL_MODULE_FILE = f'{TIYARO_HANDLER_DIR}/__init__.py'
HANDLER_MODEL_MODEL_HANDLER_FILE = f'{TIYARO_HANDLER_DIR}/{MODEL_HANDLER}.py'
HANDLER_MODEL_CUSTOM_HANDLER_FILE = f'{TIYARO_HANDLER_DIR}/{MODEL_HANDLER}_custom.py'
HANDLER_MODEL_IMAGE_CLASSIFICATION_HANDLER_FILE = f'{TIYARO_HANDLER_DIR}/{MODEL_HANDLER}_image_classification.py'
HANDLER_MODEL_TEXT_CLASSIFICATION_HANDLER_FILE = f'{TIYARO_HANDLER_DIR}/{MODEL_HANDLER}_text_classification.py'
HANDLER_MODEL_CLASS_TEMPLATES_PATTERN = f'{TIYARO_HANDLER_DIR}/{MODEL_HANDLER}_*.py'
HANDLER_MODEL_MODEL_TEST_FILE = 'tiyaro_handler_test.py'
TIYARO_INTERNAL_DIR = f'{TIYARO_HANDLER_DIR}/.tiyaro'
TIYARO_INTERNAL_STATE_FILE = f'{TIYARO_INTERNAL_DIR}/cli_state.json'
TIYARO_INTERNAL_TEST_STATE_FILE = f'{TIYARO_INTERNAL_DIR}/test_state.json'

CLASS_CUSTOM = 'custom'
CLASS_IMAGE_CLASSIFICATION = 'image-classification'
CLASS_TEXT_CLASSIFICATION = 'text-classification'

class_handler_map = {
    CLASS_IMAGE_CLASSIFICATION: HANDLER_MODEL_IMAGE_CLASSIFICATION_HANDLER_FILE,
    CLASS_TEXT_CLASSIFICATION: HANDLER_MODEL_TEXT_CLASSIFICATION_HANDLER_FILE,
    CLASS_CUSTOM: HANDLER_MODEL_CUSTOM_HANDLER_FILE
}

INIT_HELP = f'Forcefully recreates {TIYARO_HANDLER_DIR}, {HANDLER_MODEL_MODEL_TEST_FILE} templates'


# PUSH CONSTANTS
UPLOAD_ARTIFACT_DIR = './build'
GIT_SNAPSHOT_FILE_NAME = 'git_snapshot.tgz'
JSON_MANIFEST_FILE_NAME = 'manifest.json'
FILE_TO_UPLOAD = '/tmp/build.tgz'
STATUS_PAGE_URL = 'https://console.tiyaro.ai/modelstudio-publish'

CLEAN_HELP = f'Forcefully clean {TIYARO_HANDLER_DIR}, {HANDLER_MODEL_MODEL_TEST_FILE} templates'

# TEST CONSTANTS
TIYARO_TEST_VENV_PATH = './.tiyaro_test_venv'
KEYWORD_FROM_CONFIG = 'from-config'
TEST_COMMAND_ARG_PRETRAINED = '--pretrained'
TEST_COMMAND_ARG_INPUT = '--input'
TEST_COMMAND_ARG_OUTPUT = '--output'
TEST_COMMAND_ARG_VERBOSE = '--verbose'

# keys in model_handler.yml
C_PRETRAINED_FILE_OR_URL = 'pretrained_model_url'
C_MODEL_FILE_PATH = 'model_file_path'
C_REQUIREMENTS_FILE = 'reqirements_file_path'

C_MODEL_TYPE = 'model_type'
C_MODEL_CLASS = 'model_class'
C_MODEL_FRAMEWORK = 'model_framework'

# keys in test_state.json
C_TEST_INPUT = 'test_input'
C_TEST_OUTPUT = 'test_output'
C_TEST_USER_CONFIRMED = 'test_user_confirmed_output'


CMD_TIYARO_FORCE_CLEAN = 'tiyaro clean -f'
